from distutils.core import setup

setup(
    name='oskui',
    packages=['oskui'],
    version='0.13.8',
    description='Set of command line user interface tools',
    author='Oskari Vinko',
    author_email='oskari.vinko@gmail.com',
    license='MIT',
    url="https://github.com/Wing0/oskui",
    download_url="https://github.com/Wing0/oskui/archive/0.13.8.tar.gz",
    keywords=[])
